import logging

INTERFACES = ['ppp0']
RECORDS = [
    (u'@', u'example.com', u'A'),
    (u'www', u'example.com', u'A'),
]

ALIYUN_KEY_ID = ''
ALIYUN_KEY_SECRET = ''

LOG_FILE = '/var/log/aliddns.log'
LOG_FORMAT = '%(asctime)s %(levelname)s: %(message)s'
LOG_LEVEL = logging.INFO

try:
    from .local_config import *
except ImportError:
    pass



from codecs import open  # To use a consistent encoding
from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'DESCRIPTION.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='aliddns',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # http://packaging.python.org/en/latest/tutorial.html#version
    version='0.3.2',

    description='DDNS tool for domains hosted on AliYun',
    long_description=long_description,

    url='https://github.com/decentfox/aliddns',

    author='DecentFoX Studio',
    author_email='support@decentfox.com',

    license='BSD',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],

    keywords='aliyun ddns',

    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    # namespace_packages=[],

    install_requires=[
        'aliyun-python-sdk-alidns==2.0.6',
    ],

    include_package_data=True,
    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        # 'sample': ['package_data.dat'],
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages.
    # see http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[
    ],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'aliddns-ppp=aliddns.ppp:main_wrapper',
        ],
    },
)



import json
import logging
import socket
import sys
import time

import StringIO

from . import config


def _main(client, ip):
    from aliyunsdkalidns.request.v20150109.DescribeDomainRecordsRequest import *
    from aliyunsdkalidns.request.v20150109.UpdateDomainRecordRequest import *

    for rr, domain_name, type_ in config.RECORDS:
        req = DescribeDomainRecordsRequest()
        req.set_TypeKeyWord(type_)
        req.set_DomainName(domain_name)
        req.set_RRKeyWord(rr)
        records = json.loads(client.do_action_with_exception(req))['DomainRecords']['Record']

        if records:
            record = records[0]
            if record['Value'] == ip:
                logging.debug('%s.%s is already %s', rr, domain_name, ip)
            else:
                logging.info('Updating %s.%s from %s to %s',
                             rr, domain_name, record['Value'], ip)
                update = UpdateDomainRecordRequest()
                update.set_RecordId(record['RecordId'])
                update.set_RR(rr)
                update.set_Type(type_)
                update.set_Value(ip)
                client.do_action_with_exception(update)
        else:
            logging.warning('No such record %s.%s, skipping for now.',
                            rr, domain_name)


def main():
    """
    This script is called with the following arguments:
       Arg  Name                          Example
       $1   Interface name                ppp0
       $2   The tty                       ttyS1
       $3   The link speed                38400
       $4   Local IP number               12.34.56.78
       $5   Peer  IP number               12.34.56.99
       $6   Optional ``ipparam'' value    foo
    """
    # noinspection PyPackageRequirements
    from aliyunsdkcore.client import AcsClient

    if not sys.stdin.encoding:
        sys.stdin = StringIO.StringIO()
        sys.stdin.encoding = sys.getdefaultencoding()

    interface = sys.argv[1]
    if interface not in config.INTERFACES:
        return

    ip = sys.argv[4].decode('utf-8')

    logging.info('Detected %s IP change: %s', interface, ip)

    # noinspection PyUnresolvedReferences
    client = AcsClient(config.ALIYUN_KEY_ID, config.ALIYUN_KEY_SECRET, 'cn-beijing')

    tries = 0
    while True:
        try:
            tries += 1
            _main(client, ip)
            break
        except socket.error:
            if tries < 3:
                logging.warning(
                    'Network issue, try again in %s second(s).',
                    tries ** 2, exc_info=True)
                time.sleep(tries ** 2)
            else:
                logging.critical(
                    'Still no network, please set DNS manually.',
                    exc_info=True)
                break

    logging.info('Finished DDNS process.')


def main_wrapper():
    logging.basicConfig(filename=config.LOG_FILE, level=config.LOG_LEVEL,
                        format=config.LOG_FORMAT)
    # noinspection PyBroadException
    try:
        main()
    except Exception:
        logging.exception('Failed due to exception!')


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format='%(levelname)s: %(message)s')
    main()
